(*  Census Report Script
     Written by Simon Robbins
     Some changes by John Nairn
     3 SEP 2010
	 
     This script cycles through each of the census events listed in CensusDates for
     every individual in the current database or for selected individuals.

     For each event found it puts the place in a table and the cell is coloured green.
     Pale green if the event is recorded but not place stated.

     Where an individual was definitely alive but no data is held the cell is coloured amber.

     Where an individual was definitely (or probably) dead or not yet born the
     cell is coloured grey.

     Where an individual was probably alive the cell is coloured red. (The intensity of the
     colour decreases as the probability decreases. When running the script the user is
     prompted to enter a maximum age. This is the age after which individuals are
     assumed to be probably dead.

     The dates are set up to search for UK, Canadian, os US census

     John Nairn Changes:
       - accepts census event within 120 days of the dates in CensusDates
       - calculates census SDNs once at the start in a new list
       - dereferences all global lists to help speed (but small effect since lists are small)
       - screen records before calling processRecords
          note: finds a few more to to use of SND min and max rather than always the min
       - use GEDitCOM 1.3 attributes (birth date, birth SDN, birth SDN max, birth place, and same for death and event)
       - tried to set "end of list" in some loops rather then setting an item
       - unRpt to a list rather than a string (converted to string at output)
           note: this was most important changed for speed
       - changed name to "British Census Report"
       - changed some properties to globals when they are always initialized anyway
       - added check for no records found
       - added option for Canada and US
	- added key

     Hint:
        You probably only want to report on individual known to be in the country relevant
		to the selected census information. A good way to do this is
	 1. Open search window and search for "Any Place" "contains" the country name
	 2. When found, save all search hits in a new album
	 3. Select all records in that album, run this script, and chose "Selected" individuals
	 
	 
	 Further changes by Simon Robbins (Oct 2014)
	 	-addition of DataTables javascript to give nice scrolling with fixed headers and columns
	 	this also enables sorting of the table after it has been produced and filtering by name etc.
		
	Additional changes by Mick Reed (July 2022) to include 1921 Census for England and Wales

*)

-- globals
property scriptName : "UK-US-Canadian Census Report"
property BritCensusDates : {"6 JUN 1841", "30 MAR 1851", "7 APR 1861", "2 APR 1871", "3 APR 1881", "5 APR 1891", "31 MAR 1901", "2 APR 1911", "19 JUN 1921"}
property BritCensusNames : {"1841", "1851", "1861", "1871", "1881", "1891", "1901", "1911", "1921"}
property CanCensusDates : {"12 JAN 1852", "14 JAN 1861", "2 APR 1871", "4 APR 1881", "6 APR 1891", "31 MAR 1901", "24 JUN 1906", "1 JUN 1911", "1 JUN 1916", "1 JUN 1921"}
property CanCensusNames : {"1851 (delayed to 1852)", "1861", "1871", "1881", "1891", "1901", "1906 (AB, MB & SK)", "1911", "1916 (AB, MB & SK)", "1921"}
property USCensusDates : {"August 2, 1790", "August 4, 1800", "August 6, 1810", "August 7, 1820", "June 1, 1830", "June 1, 1840", "June 1, 1850", "June 1, 1860", "June 1, 1870", "June 1, 1880", "June 2, 1890", "June 1, 1900", "April 15, 1910", "January 5, 1920", "April 1, 1930", "1 APR 1940"}
property USCensusNames : {"1790", "1800", "1810", "1820", "1830", "1840", "1850", "1860", "1870", "1880", "1890", "1900", "1910", "1920", "1930", "1940"}
property speculative : {"spec1", "spec2", "spec3", "spec4", "spec5", "spec6", "spec7", "spec8", "spec9", "spec10", "spec11"}
global unRpt, minbdtesdn, maxbdtesdn, minddtesdn, maxddtesdn, MaxAge
global NumCENS, CensusSDNsRef, CensusSDNs, colnum, CensusDates, CensusNames
global ColData, ColDataRef, ColWithData, ColWithDataRef, colour, colourRef, speculativeRef

-- pick census dates and names by country
set CensusDates to BritCensusDates
set CensusNames to BritCensusNames

-- if no document is open then quit
if CheckAvailable(scriptName, 1.5) is false then return

tell application "GEDitCOM II"
	-- which census years?
	try
		set WindowDef to my SetWindowDef()
		
		tell application "Finder"
			set Options to my pashua_run(WindowDef, "", "")
		end tell
		activate
		
		
		if cb of Options is "1" then
			return
		end if
		
		set cenGroup to cenGroup of Options
		set MaxAge to MaxAge of Options
		set whichOnes to whichOnes of Options
	on error
		-- which census years?
		set cenOptions to {"US Censuses 1790 to 1940", "UK Censuses 1841 to 1921", "Canadian Census 1851 to 1921"}
		set thePrompt to "Select census group and then click button for which individual to include in the report:"
		set r to user choice prompt thePrompt list items cenOptions buttons {"All", "Selected", "Cancel"} title scriptName
		set whichOnes to item 1 of r
		if whichOnes is "Cancel" then return
		set cenGroup to item 1 of item 2 of r
		
		-- Max Age option (must be integer 1 or higher)
		set thePrompt to "Enter maximum age after which persons with no death date are assumed dead"
		set r to user input prompt thePrompt initial text "80" title scriptName buttons {"OK", "Cancel"}
		if item 1 of r is "Cancel" then return
		set MaxAge to item 2 of r
		
	end try
	
	if whichOnes is "Cancel" then return
	--	set cenGroup to item 1 of item 2 of r
	if cenGroup is "US Censuses 1790 to 1940" then
		set CensusDates to USCensusDates
		set CensusNames to USCensusNames
	else if cenGroup is "UK Censuses 1841 to 1921" then
		set CensusDates to BritCensusDates
		set CensusNames to BritCensusNames
	else
		set CensusDates to CanCensusDates
		set CensusNames to CanCensusNames
	end if
	
	try
		set MaxAge to MaxAge as integer
		if MaxAge < 1 or MaxAge > 110 then error
	on error
		user option title "The maximum age must be a number and be between 1 and 110." buttons {"OK"} message "Please try the script again"
		return
	end try
	
	--calculate date ranges required in report and select records
	set colnum to (count of CensusDates)
	set CensusSDNs to {}
	repeat with i from 1 to colnum
		set end of CensusSDNs to item 1 of (sdn range full date (item i of CensusDates))
	end repeat
	set CensusSDNsRef to a reference to CensusSDNs
	set ColDataRef to a reference to ColData
	set ColWithDataRef to a reference to ColWithData
	set colourRef to a reference to colour
	set speculativeRef to a reference to speculative
	
	-- find date ranges from exact census dates
	set minbdtesdn to (item 1 of CensusSDNsRef) - (365 * MaxAge)
	set maxbdtesdn to item colnum of CensusSDNsRef
	set minddtesdn to item 1 of CensusSDNsRef
	set maxddtesdn to (item colnum of CensusSDNsRef) + (365 * MaxAge)
	
	
	if whichOnes is "All" then
		-- do all individuals
		tell front document
			set recs to every individual whose 
				(birth SDN max > minbdtesdn and birth SDN < maxbdtesdn) or 
				(death SDN max > minddtesdn and death SDN < maxddtesdn)
			--set recs to every individual whose (birth SDN is not 0 or death SDN is not 0) and 
			--	((birth SDN max > minbdtesdn and birth SDN < maxbdtesdn) or birth SDN = 0) and 
			--	((death SDN max > minddtesdn and death SDN < maxddtesdn) or death SDN = 0) 
			--		or (birth SDN max > minddtesdn and birth SDN < maxbdtesdn) 
			--	or (death SDN max > minddtesdn and death SDN < maxbdtesdn)
			
		end tell
		--return (count of recs) as string
	else
		-- do just the selected records
		set selRecs to selected records of front document
		set numSel to count of selRecs
		set recs to {}
		repeat with x from 1 to numSel
			set oneRec to item x of selRecs
			if record type of oneRec is "INDI" then
				tell oneRec
					if (birth SDN max > minbdtesdn and birth SDN < maxbdtesdn) or 
						(death SDN max > minddtesdn and death SDN < maxddtesdn) then
						set end of recs to oneRec
					end if
					--if (birth SDN is not 0 or death SDN is not 0) and 
					--	((birth SDN max > minbdtesdn and birth SDN < maxbdtesdn) or birth SDN = 0) and 
					--	((death SDN max > minddtesdn and death SDN < maxddtesdn) or death SDN = 0) 
					--		or (birth SDN max > minddtesdn and birth SDN < maxbdtesdn) 
					--	or (death SDN max > minddtesdn and death SDN < maxbdtesdn) then
					--	set end of recs to oneRec
					--end if
				end tell
			end if
		end repeat
	end if
end tell

-- call method to process the records
tell application "GEDitCOM II"
	if (count of recs) is 0 then
		user option title "No selected records were found that could have census events for the selected censuses" buttons {"OK"}
		return
	end if
	
	set fName to name of front document
	tell front document
		
		my processRecords(recs, cenGroup)
		
	end tell
end tell

-- prepare report
tell application "Finder"
	set screen_size to bounds of window of desktop
	set screen_width to item 3 of screen_size
	set screen_height to item 4 of screen_size
end tell


set rpt to "<div>" & return
set rpt to rpt & "<!DOCTYPE html>" & return
set rpt to rpt & "<html>" & return
set rpt to rpt & "<head>" & return

-- If the DataTables javascript has not been downloaded, comment out the next two lines and uncomment the two below that
--set rpt to rpt & "    <link rel=\"stylesheet\" type=\"text/css\" href=\"/Applications/DataTables-1.10.3/media/css/jquery.dataTables.css\">" & return
--set rpt to rpt & "    <link rel=\"stylesheet\" type=\"text/css\" href=\"/Applications/DataTables-1.10.3/extensions/FixedColumns/css/dataTables.fixedColumns.css\">" & return

-- If the DataTables javascript has been downloaded, comment out the next two lines and uncomment the two above
set rpt to rpt & "    <link rel=\"stylesheet\" type=\"text/css\" href=\"https://datatables.net/release-datatables/media/css/jquery.dataTables.css\">" & return
set rpt to rpt & "    <link rel=\"stylesheet\" type=\"text/css\" href=\"https://datatables.net/release-datatables/extensions/FixedColumns/css/dataTables.fixedColumns.css\">" & return

set rpt to rpt & "    <style type=\"text/css\" class=\"init\">" & return

set rpt to rpt & "    /* Ensure that the table scrolls and change some of the settings in the default report.css*/" & return
set rpt to rpt & "    th, td { white-space: nowrap; " & return
set rpt to rpt & "    color:black !important;" & return
set rpt to rpt & "    border-left: 0 !important;" & return
set rpt to rpt & "    border-bottom: 0 !important;" & return
set rpt to rpt & "    }" & return
set rpt to rpt & "    table { border: 0 !important; " & return
set rpt to rpt & "    }" & return
set rpt to rpt & "    div.dataTables_wrapper {" & return
set rpt to rpt & "        width: 100%;" & return
set rpt to rpt & "        margin: 0 auto;" & return

set rpt to rpt & "    }" & return

set rpt to rpt & "    /* Ensure that cell colours don't change when sorting on a year column*/" & return
set rpt to rpt & "    #lightgreen {" & return
set rpt to rpt & "        background-color: #33ff33;" & return
set rpt to rpt & "    }" & return
set rpt to rpt & "    #darkgreen {" & return
set rpt to rpt & "        background-color: #009900;" & return
set rpt to rpt & "    }" & return
set rpt to rpt & "    #amber {" & return
set rpt to rpt & "        background-color: #ff9900;" & return
set rpt to rpt & "    }" & return
set rpt to rpt & "    #grey {" & return
set rpt to rpt & "        background-color: #606060;" & return
set rpt to rpt & "    }" & return
set rpt to rpt & "    #spec1 {" & return
set rpt to rpt & "        background-color: #ff1919;" & return
set rpt to rpt & "    }" & return
set rpt to rpt & "    #spec2 {" & return
set rpt to rpt & "        background-color: #ff3333;" & return
set rpt to rpt & "    }" & return
set rpt to rpt & "    #spec3 {" & return
set rpt to rpt & "        background-color: #ff4d4d;" & return
set rpt to rpt & "    }" & return
set rpt to rpt & "    #spec4 {" & return
set rpt to rpt & "        background-color: #ff6666;" & return
set rpt to rpt & "    }" & return
set rpt to rpt & "    #spec5 {" & return
set rpt to rpt & "        background-color: #ff8080;" & return
set rpt to rpt & "    }" & return
set rpt to rpt & "    #spec6 {" & return
set rpt to rpt & "        background-color: #ff9999;" & return
set rpt to rpt & "    }" & return
set rpt to rpt & "    #spec7 {" & return
set rpt to rpt & "        background-color: #ffb2b2;" & return
set rpt to rpt & "    }" & return
set rpt to rpt & "    #spec8 {" & return
set rpt to rpt & "        background-color: #ffcccc;" & return
set rpt to rpt & "    }" & return
set rpt to rpt & "    #spec9 {" & return
set rpt to rpt & "        background-color: #ffdddd;" & return
set rpt to rpt & "    }" & return
set rpt to rpt & "    #spec10 {" & return
set rpt to rpt & "        background-color: #ffeeee;" & return
set rpt to rpt & "    }" & return
set rpt to rpt & "    #spec11 {" & return
set rpt to rpt & "        background-color: #fff9f9;" & return
set rpt to rpt & "    }" & return


set rpt to rpt & "</style>" & return

-- If the DataTables javascript has not been downloaded, comment out the next three lines and uncomment the three below that
--set rpt to rpt & "   <script type=\"text/javascript\" language=\"javascript\" src=\"/Applications/DataTables-1.10.3/media/js/jquery.js\"></script>" & return
--set rpt to rpt & "    <script type=\"text/javascript\" language=\"javascript\" src=\"/Applications/DataTables-1.10.3/media/js/jquery.dataTables.js\"></script>" & return
--set rpt to rpt & "    <script type=\"text/javascript\" language=\"javascript\" src=\"/Applications/DataTables-1.10.3/extensions/FixedColumns/js/dataTables.fixedColumns.js\"></script>" & return

-- If the DataTables javascript has  been downloaded, comment out the next three lines and uncomment the three above
set rpt to rpt & "    <script type=\"text/javascript\" language=\"javascript\" src=\"https://datatables.net/release-datatables/media/js/jquery.js\"></script>" & return
set rpt to rpt & "    <script type=\"text/javascript\" language=\"javascript\" src=\"https://datatables.net/release-datatables/media/js/jquery.dataTables.js\"></script>" & return
set rpt to rpt & "    <script type=\"text/javascript\" language=\"javascript\" src=\"https://datatables.net/release-datatables/extensions/FixedColumns/js/dataTables.fixedColumns.js\"></script>" & return

set rpt to rpt & "    <script type=\"text/javascript\" language=\"javascript\" class=\"init\">" & return


set rpt to rpt & "$(document).ready(function() {" & return
set rpt to rpt & "    var table = $('#example').DataTable( {" & return
set rpt to rpt & "columnDefs: [" & return

set rpt to rpt & "      {type: 'non-empty-string', targets: 2}," & return
set rpt to rpt & "      {type: 'non-empty-string', targets: 3}," & return
set colnum to (count of CensusDates)
repeat with i from 1 to colnum
	set rpt to rpt & "      {type: 'non-empty-string', targets: " & (i + 3) & "}"
	if i is not colnum then
		set rpt to rpt & "," & return
	else
		set rpt to rpt & return
	end if
end repeat

set rpt to rpt & "   ]," & return
set rpt to rpt & "        scrollY:        \"" & screen_height - 525 & "px\"," & return
set rpt to rpt & "        scrollX:        true," & return
set rpt to rpt & "        scrollCollapse: true," & return
set rpt to rpt & "        paging:         false" & return
set rpt to rpt & "    } );" & return
set rpt to rpt & "    new $.fn.dataTable.FixedColumns( table, {" & return
set rpt to rpt & "        leftColumns: 4" & return
set rpt to rpt & "    } );" & return
set rpt to rpt & "jQuery.extend( jQuery.fn.dataTableExt.oSort, {" & return
set rpt to rpt & "    \"non-empty-string-asc\": function (str1, str2) {" & return
set rpt to rpt & "        if(str1 == \"\")" & return
set rpt to rpt & "            return 1;" & return
set rpt to rpt & "        if(str2 == \"\")" & return
set rpt to rpt & "            return -1;" & return
set rpt to rpt & "        return ((str1 < str2) ? -1 : ((str1 > str2) ? 1 : 0));" & return
set rpt to rpt & "    }," & return
set rpt to rpt & return
set rpt to rpt & "    \"non-empty-string-desc\": function (str1, str2) {" & return
set rpt to rpt & "        if(str1 == \"\")" & return
set rpt to rpt & "            return 1;" & return
set rpt to rpt & "       if(str2 == \"\")" & return
set rpt to rpt & "           return -1;" & return
set rpt to rpt & "       return ((str1 < str2) ? 1 : ((str1 > str2) ? -1 : 0));" & return
set rpt to rpt & "   }" & return
set rpt to rpt & "} );" & return
set rpt to rpt & "} );" & return


set rpt to rpt & "    </script>" & return
set rpt to rpt & "</head>" & return

set rpt to rpt & "<body>" & return
set rpt to rpt & "    <div width=\"100%\">" & return
set rpt to rpt & "        <section>" & return
set rpt to rpt & "          <div>" & return
set rpt to rpt & "<h1>Census Events in " & cenGroup & " for " & fName & "</h1>" & return



set rpt to rpt & "&nbsp;&nbsp;&nbsp;Table Key" & return
set rpt to rpt & "<table>" & return
set rpt to rpt & "<tr><td colspan='8' bgcolor=#009900></td><td >Census detail has been recorded with place</td></tr>" & return
set rpt to rpt & "<tr><td colspan='8' bgcolor=#33ff33></td><td >Census detail has been recorded but place field is blank</td></tr>" & return
set rpt to rpt & "<tr><td colspan='8' bgcolor=#ff9900></td><td >Individual was alive but no census data recorded</td></tr>" & return
set rpt to rpt & "<tr><td bgcolor=#ff1919 width=10px></td>" & return
set rpt to rpt & "          <td bgcolor=#ff3333 width=10px></td>" & return
set rpt to rpt & "          <td bgcolor=#ff4d4d width=10px></td>" & return
set rpt to rpt & "          <td bgcolor=#ff6666 width=10px></td>" & return
set rpt to rpt & "          <td bgcolor=#ff8080 width=10px></td>" & return
set rpt to rpt & "          <td bgcolor=#ff9999 width=10px></td>" & return
set rpt to rpt & "          <td bgcolor=#ffb2b2 width=10px></td>" & return
set rpt to rpt & "          <td bgcolor=#ffcccc width=10px></td><td >Individual may have been alive but no census data recorded (the darker the shade, the more likely they were alive)</td></tr>" & return
set rpt to rpt & "<tr><td colspan='8' bgcolor=#606060></td><td >Individual was not alive at the time of the census</td></tr>" & return
set rpt to rpt & "</table>" & return
set rpt to rpt & "</div>" & return
set rpt to rpt & "<br>" & return
set rpt to rpt & "<br>" & return


set rpt to rpt & "<table id=\"example\" class=\"stripe row-border order-column\" cellspacing=\"0\" width=\"100%\">" & return




set rpt to rpt & "<thead>" & return
set rpt to rpt & "<tr>" & return
set rpt to rpt & "<th>No.</th>" & return
set rpt to rpt & "<th>Name</th>" & return
set rpt to rpt & "<th>Birth</th>" & return
set rpt to rpt & "<th>Death</th>" & return
set colnum to (count of CensusDates)
repeat with i from 1 to colnum
	set rpt to rpt & "<th>" & item i of CensusNames & "</th>" & return
end repeat
set rpt to rpt & "</tr>" & return
set rpt to rpt & "</thead>" & return


set rpt to rpt & "<tbody>" & return


set rpt to rpt & (unRpt as string) & return

set rpt to rpt & "</tbody>" & return
set rpt to rpt & "</table>" & return
set rpt to rpt & "</body>" & return
set rpt to rpt & "</html>" & return
set rpt to rpt & "</div>" & return



-- add report to document and open it
tell application "GEDitCOM II"
	tell front document
		set newreport to make new report with properties {name:"Census Report", body:rpt}
		show browser of newreport
	end tell
	
	(*
	tell application "GEDitCOM II"
		set br to bounds of front window
		if screen_width > 1450 then
			set item 1 of br to (screen_width - 1450) / 2
			set item 3 of br to screen_width - ((screen_width - 1450) / 2)
		else
			set item 1 of br to 22
			set item 3 of br to screen_width - 22
		end if
		set item 2 of br to 50
		set item 4 of br to screen_height - 50
		set bounds of front window to br
	end tell
	*)
	
	tell application "GEDitCOM II"
		set br to bounds of front window
		set item 1 of br to 22
		set item 3 of br to screen_width - 22
		--end if
		set item 2 of br to 50
		set item 4 of br to screen_height - 40
		
		set bounds of front window to br
	end tell
	
end tell

return



on processRecords(recSet, cenGroup)
	tell application "GEDitCOM II"
		set fractionStepSize to 0.01 -- progress reporting interval
		set nextFraction to fractionStepSize -- progress reporting interval
		set unRpt to {}
		set linenum to 0
		set num to number of items in recSet
		repeat with r from 1 to num
			set IndiRec to item r of recSet
			
			
			tell IndiRec
				set bdate to birth SDN
				set ddate to death SDN
				
				set ColDataRef to {}
				repeat with x from 1 to colnum
					set end of ColDataRef to ""
				end repeat
				set cens to find structures tag "CENS" output "references"
				set cnum to number of items in cens
				
				
				repeat with c from 1 to cnum
					set crec to item c of cens
					set cimgind to ""
					set cnoteind to ""
					tell crec
						set cdate to (event SDN + event SDN max) / 2
						set cplac to event place
						
						set censour to find structures tag "SOUR" output "references"
						set snum to number of items in censour
						
						repeat with s from 1 to snum
							set srec to item s of censour
							tell srec
								set cimg to evaluate expression "OBJE._FILE"
								if "Census" is not in cimg then
									set cimgind to "<span style='color:white'> [i]</span>"
								end if
								
								set cnote to evaluate expression "NOTE.CONC"
								if "Census Entry" is not in cnote then
									set cnoteind to "<span style='color:white'> [n]</span>"
								end if
								
								(*
								This next section is designed to decide whether or not
								to include a census event in the report based on whether the
								user has selected UK, Canadian or US censuses and the name
								of the cited source record. Using date alone is unreliable
								as even if the date of the event needs to be exact it will fail
								as the 1871 and 1901 censuses in the UK and Canada were on the same day.
								
								This fix will only work if the census event is attached to a source 
								with a recognisable name and may need adjusting for other users databases.
								*)
								tell front document of application "GEDitCOM II"
									set sName to name of gedcomRecord id (contents of srec)
								end tell
								
								if cenGroup contains "US " and sName contains "US " then
									set conf to "Y"
								else if cenGroup contains "Canadian" and sName contains "Canadian" then
									set conf to "Y"
								else if cenGroup contains "UK" and (sName contains "18" or sName contains "19") then
									set conf to "Y"
								else
									set conf to "N"
								end if
								
							end tell
						end repeat
						
						if conf is "Y" then
							repeat with x from 1 to colnum
								set cdiff to cdate - (item x of CensusSDNsRef)
								if cdiff > -120 and cdiff < 120 then
									if cplac is not "" then
										set item x of ColDataRef to cplac
									else
										set item x of ColDataRef to "Not entered"
									end if
									set item x of ColDataRef to item x of ColDataRef & cnoteind & cimgind
								end if
							end repeat
						end if
						
					end tell
				end repeat
				
				-- Work out which columns have data
				set ColWithDataRef to {}
				repeat with x from 1 to colnum
					if item x of ColDataRef is not "" then
						set end of ColWithDataRef to x
					else
						set end of ColWithDataRef to ""
					end if
				end repeat
				
				
				--Calculate cell colour
				
				set colourRef to {}
				set coloursortref to {}
				repeat with x from 1 to colnum
					set thisColour to "ffffff"
					-- If person not yet born or already died at time of census colour cell grey
					if (bdate > item x of CensusSDNsRef) or (ddate < item x of CensusSDNsRef) then
						set thisColour to "grey"
						set coloursort to ""
					end if
					
					--If census record exists but no place entered colour cell light green
					if item x of ColDataRef is "Not Entered" then
						set thisColour to "lightgreen"
						set coloursort to "<div style=\"visibility: hidden; height: 0px\">" & "02lightgreen" & "</div>"
					end if
					
					--If census record exists and place entered colour cell dark green
					if item x of ColDataRef is not "Not Entered" and item x of ColDataRef is not "" then
						set thisColour to "darkgreen"
						set coloursort to "<div style=\"visibility: hidden; height: 0px\">" & "01darkgreen" & "</div>"
					end if
					
					--If census record does not exist but we know it must do colour cell amber
					if (bdate is not 0 or ddate is not 0) then
						if item x of ColDataRef is "" and (x - 1 is in ColWithDataRef or x - 2 is in ColWithDataRef or x - 3 is in ColWithDataRef or x - 4 is in ColWithDataRef or x - 5 is in ColWithDataRef or x - 6 is in ColWithDataRef or x - 7 is in ColWithDataRef or bdate < item x of CensusSDNsRef) and (x + 1 is in ColWithDataRef or x + 2 is in ColWithDataRef or x + 3 is in ColWithDataRef or x + 4 is in ColWithDataRef or x + 5 is in ColWithDataRef or x + 6 is in ColWithDataRef or x + 7 is in ColWithDataRef or ddate > item x of CensusSDNsRef) then
							set thisColour to "amber"
							set coloursort to "<div style=\"visibility: hidden; height: 0px\">" & "03amber" & "</div>"
						end if
					end if
					
					if item x of ColDataRef is "" and bdate is 0 and (x - 1 is not in ColWithDataRef and x - 2 is not in ColWithDataRef and x - 3 is not in ColWithDataRef and x - 4 is not in ColWithDataRef and x - 5 is not in ColWithDataRef and x - 6 is not in ColWithDataRef and x - 7 is not in ColWithDataRef) then
						set y to round (((ddate) - (item x of CensusSDNsRef)) / 3652.5) rounding up
						if y > MaxAge / 10 or y < 1 then
							set thisColour to "grey"
							set coloursort to ""
						else
							set thisColour to item y of speculativeRef
							set coloursort to "<div style=\"visibility: hidden; height: 0px\">" & "04" & item y of speculativeRef & "</div>"
						end if
					end if
					
					if item x of ColDataRef is "" and ddate is 0 and (x + 1 is not in ColWithDataRef and x + 2 is not in ColWithDataRef and x + 3 is not in ColWithDataRef and x + 4 is not in ColWithDataRef and x + 5 is not in ColWithDataRef and x + 6 is not in ColWithDataRef and x + 7 is not in ColWithDataRef) then
						set y to round (((item x of CensusSDNsRef) - (bdate)) / 3652.5) rounding up
						if y > MaxAge / 10 or y < 1 then
							set thisColour to "grey"
							set coloursort to ""
						else
							set thisColour to item y of speculativeRef
							set coloursort to "<div style=\"visibility: hidden; height: 0px\">" & "04" & item y of speculativeRef & "</div>"
						end if
					end if
					
					set end of colourRef to thisColour
					set end of coloursortref to coloursort
				end repeat
				
				set linenum to linenum + 1
				if bdate is not 0 then
					set thisBirt to "<div style=\"visibility: hidden; height: 0px\">" & bdate & "</div>" & (date format full date birth date)
				else
					set thisBirt to ""
				end if
				if ddate is not 0 then
					set thisDeat to "<div style=\"visibility: hidden; height: 0px\">" & ddate & "</div>" & (date format full date death date)
				else
					set thisDeat to ""
				end if
				set columnhtml to {}
				repeat with x from 1 to colnum
					set end of columnhtml to "<td id=\"" & item x of colourRef & "\">" & item x of coloursortref & item x of ColDataRef & "</td>"
				end repeat
				set columnhtmlstring to columnhtml as string
				
				set end of unRpt to "<tr><td>" & linenum & ".</td><td><a href='//" & (id of IndiRec) & "'>" & (name of IndiRec) & "</a></td><td>" & thisBirt & "</td><td>" & thisDeat & "</td>" & columnhtmlstring & "</tr>" & return
				
				
			end tell
			
			
			
			-- time for progress
			set fractionDone to r / num
			if fractionDone > nextFraction then
				notify progress fraction fractionDone
				set nextFraction to nextFraction + fractionStepSize
			end if
		end repeat
	end tell
end processRecords




(* Activate GEDitCOM II (if needed) and verify acceptable
     version is running and a document is open. Return true
     or false if script can run.
*)
on CheckAvailable(sName, vNeed)
	tell application "GEDitCOM II"
		activate
		if versionNumber < vNeed then
			user option title "The script '" & sName & 
				"' requires GEDitCOM II, Version " & vNeed & 
				" or newer" message "Please upgrade and try again." buttons {"OK"}
			return false
		end if
		if number of documents is 0 then
			user option title "The script '" & sName & 
				
					"' requires a document to be open" message 
				"Please open a document and try again." buttons {"OK"}
			return false
		end if
	end tell
	return true
end CheckAvailable


on SetWindowDef()
	-- Create a window definition string which will be used to
	-- tell Pashua what types of GUI elements you need and
	-- which default values (if any) should be used
	
	-- Get the GEDitCOM icon to add to the dialog
	
	set diskPath to (path to startup disk as string)
	set iconPath to (diskPath & "Applications:GEDitCOM II.app:Contents:Resources:AppProIcon.icns")
	
	
	-- This sets up the bits which are common to all event dialogs regardless of type
	set WindowDef to "
# Set transparency: 0 is transparent, 1 is opaque
*.transparency=1

# Make sure the window 'floats' on top
*.floating=1

# Set window title
*.title = Census Report

#Add the GEDitCOM icon
img.type = image
img.maxwidth = 65
img.path = " & POSIX path of iconPath & return & "


# Add a cancel button with default label
cb.type=cancelbutton

img.x = 0
img.y = 115

#radio buttons to select Census
cenGroup.type = radiobutton
cenGroup.label = Select the census for the report
cenGroup.option = US Censuses 1790 to 1940
cenGroup.option = UK Censuses 1841 to 1921
cenGroup.option = Canadian Censuses 1851 to 1921
cenGroup.default = US Censuses 1790 to 1940"
	
	set WindowDef to WindowDef & "

cenGroup.y=85
cenGroup.x=95

#radio buttons to select all or selected
whichOnes.type = radiobutton
whichOnes.label = Individuals to include
whichOnes.option = All
whichOnes.option = Selected
whichOnes.default = All
whichOnes.y =104
whichOnes.x=330

MaxAge.type = textfield
MaxAge.label = Enter maximum age after which persons with no death date are assumed dead
MaxAge.default = 80
MaxAge.width = 50
MaxAge.y =25
MaxAge.x=0
"
	return WindowDef
	
end SetWindowDef

-- Glue code for interfacing from AppleScript to Pashua. Written by
-- Carsten Blm <carsten@bluem.net>, 10/2003-01/2006, with improvements
-- contributed by Eddy Roosnek and Hans Haesler. You can use or modify
-- this handler any way you like in your own scripts.
-- Argument 1: Configuration string / window description
-- Argument 2: Encoding to use; if empty, Pashua assumes "macroman"
-- Argument 3: Folder that contains Pashua.app; if empty, default locations are searched
on pashua_run(WindowDef, encoding, appdir)
	
	-- Create path for temporary file
	set AppleScript's text item delimiters to ""
	set tmpfile to ((path to temporary items folder as string) & "Pashua_" & (characters 3 thru end of ((random number) as string)) as string)
	
	-- Write temporary file and fill it with the configuration string
	set fhandle to open for access tmpfile with write permission
	write (WindowDef as string) to fhandle
	close access fhandle
	
	-- Get temporary file's POSIX path
	set posixtmpfile to POSIX path of tmpfile
	
	
	set diskPath to (path to startup disk as string)
	set userPath to path to "cusr" as string
	set myself to (path to me as string)
	tell application "Finder" to set myParentPath to (container of alias myself as string)
	
	-- Try to find Pashua application
	tell application "Finder"
		
		-- Try to find it in the directory supplied as argument to this handler
		if appdir is not "" then
			if last character of appdir = ":" then
				set dirsep to ""
			else
				set dirsep to ":"
			end if
			if item (appdir & dirsep & "Pashua.app") exists then
				set pashua to appdir & dirsep & "Pashua.app:"
			end if
			-- Try to find it in Modified Default Format
		else if item (userPath & "Library:Application Support:GEDitCOM II:Formats:Modified Default Format.gfrmt:Pashua.app") exists then
			set pashua to (userPath & "Library:Application Support:GEDitCOM II:Formats:Modified Default Format.gfrmt:Pashua.app:")
			-- Try to find it in this script application bundle
		else if item (myself & "Contents:MacOS:Pashua") exists then
			set pashua to myself
			-- Try to find it in this script's parent's path
		else if item (myParentPath & "Pashua.app") exists then
			set pashua to (myParentPath & "Pashua.app:")
			-- Try to find it in global application folder
		else if item (diskPath & "Applications:Pashua.app") exists then
			set pashua to (diskPath & "Applications:Pashua.app:")
			-- Try to find it in user's application folder
		else if item (userPath & "Applications:Pashua.app") exists then
			set pashua to (userPath & "Applications:Pashua.app:")
			
			(*else
			--display dialog "Error" message "I can't find the Pashua application which is needed for this script." & return & return & "It looks like Pashua is neither in one of the standard locations nor in the folder this AppleScript is in." & return & return & "Pashua can be downloaded from:" & return & "http://www.bluem.net/en/mac/pashua/" buttons {"OK"} default button 1 with icon stop
			tell application "GEDitCOM II"
				user option title "Error" message "I can't find the Pashua application which is needed for this script." & return & return & "It looks like Pashua is neither in one of the standard locations nor in the folder this AppleScript is in." & return & return & "Pashua can be downloaded from:" & return & "http://www.bluem.net/en/mac/pashua/" buttons {"OK"}
			end tell
			return -1*)
		end if
	end tell
	
	-- Append binary position inside app bundle to "regular" path
	-- and convert path from HFS to POSIX representation
	set pashuabinary to (POSIX path of pashua) & "Contents/MacOS/Pashua"
	
	-- Optionally, define the encoding as command-line argument
	if encoding = "" then
		set encodingArg to ""
	else
		set encodingArg to "-e " & encoding & " "
	end if
	
	-- Execute pashua and get the string returned
	set pashuaCall to "'" & pashuabinary & "' " & encodingArg & "'" & posixtmpfile & "';sleep 0.001"
	
	set pashuaResult to do shell script (pashuaCall)
	
	
	-- Delete the temporary file
	do shell script "rm -r " & posixtmpfile
	
	
	-- Check whether the dialog was submitted at all.
	-- If this is not the case, return an empty list
	if pashuaResult = "" then
		return {}
	end if
	
	-- Parse the result
	set AppleScript's text item delimiters to return
	set resultLines to text items of pashuaResult
	set AppleScript's text item delimiters to ""
	set recordComponents to {}
	repeat with currentLine in resultLines
		set eqpos to offset of "=" in currentLine
		if eqpos is not 0 then
			set varKey to word 1 of currentLine
			try
				set varValue to (text (eqpos + 1) thru end of currentLine)
				-- Quote any quotation marks in varValue with a backslash.
				-- The proper way to do this would be a handler, but as
				-- all code for interfacing to Pashua should be as compact
				-- as possible, we rather do it inline
				set AppleScript's text item delimiters to "\""
				set textItems to every text item of varValue
				set AppleScript's text item delimiters to "\\\""
				set varValue to textItems as string
				set AppleScript's text item delimiters to ""
			on error
				set varValue to ""
			end try
			copy (varKey & ":\"" & varValue & "\"") to end of recordComponents
		end if
	end repeat
	
	-- Return the record we read from the tmpfile
	set AppleScript's text item delimiters to ", "
	set resultList to (run script "return {" & (recordComponents as string) & "}")
	set AppleScript's text item delimiters to {""}
	return resultList
	
end pashua_run
